"""Constants for the pyvelop module."""

# region #-- generic details --#
_PACKAGE_NAME: str = "pyvelop"
_PACKAGE_AUTHOR: str = "uvjim"
_PACKAGE_VERSION: str = "2022.9.4"
# endregion

DEF_REDACT: str = "**REDACTED**"
